@@ -7,12 +7,19 @@ import numpy as np
 import os
 import time
 import yaml
+from astropy.time import TimeDelta
 from glob import glob
+from pyoperators import DenseOperator, HomothetyOperator, Rotation3dOperator
 from pyoperators.utils import ifirst
-from pysimulators import Acquisition
+from pysimulators import (
+    Acquisition, CartesianEquatorial2HorizontalOperator,
+    CartesianGalactic2EquatorialOperator,
+    CartesianHorizontal2EquatorialOperator,
+    CartesianEquatorial2GalacticOperator)
+from pysimulators.interfaces.healpy import HealpixConvolutionGaussianOperator
+
 from .calibration import QubicCalibration
 from .instrument import QubicInstrument
-from .operators import HealpixConvolutionGaussianOperator
 
 __all__ = ['QubicAcquisition']
 
@@ -51,8 +58,12 @@ class QubicAcquisition(Acquisition):
             raise NotImplementedError('Module names not fixed yet.')
         Acquisition.__init__(self, instrument, pointing, block_id=block_id,
                              selection=selection)
+        # XXX HACK
+        from .pointings import QubicPointing
+        if isinstance(pointing, QubicPointing):
+            self.pointing = pointing
 
-    def get_pointing_hitmap(self, nside=None):
+    def get_hitmap(self, nside=None):
         """
         Return a healpy map whose values are the number of times a pointing
         hits the pixel.
@@ -60,12 +71,37 @@ class QubicAcquisition(Acquisition):
         """
         if nside is None:
             nside = self.instrument.sky.nside
-        hit = np.zeros(12 * nside**2)
-        theta, phi = self.pointing[..., 0], self.pointing[..., 1]
-        ipixel = hp.ang2pix(nside, np.radians(theta), np.radians(phi))
-        for i in ipixel:
-            hit[i] += 1
-        return hit
+        ipixel = self.pointing.tohealpix(nside)
+        npixel = 12 * nside**2
+        return np.histogram(ipixel, bins=npixel, range=(0, npixel))[0]
+
+    def get_rotation(self):
+        """
+        Return the instrument-to-galactic rotation matrix.
+
+        """
+        p = self.pointing
+        time = p.date_obs + TimeDelta(p.time, format='sec')
+        r = CartesianEquatorial2GalacticOperator() * \
+            Rotation3dOperator("ZY'Z''", p.azimuth, 90 - p.elevation, p.pitch,
+                               degrees=True) * \
+            CartesianHorizontal2EquatorialOperator('NE', time, p.latitude,
+                                                   p.longitude)
+        return r
+
+    def get_rotation_g2i(self):
+        """
+        Return the galactic-to-instrument rotation matrix.
+
+        """
+        p = self.pointing
+        time = p.date_obs + TimeDelta(p.time, format='sec')
+        r = Rotation3dOperator("ZY'Z''", p.azimuth, 90 - p.elevation, p.pitch,
+                               degrees=True).T * \
+            CartesianEquatorial2HorizontalOperator('NE', time, p.latitude,
+                                                   p.longitude) * \
+            CartesianGalactic2EquatorialOperator()
+        return r
 
     def get_convolution_peak_operator(self, fwhm=np.radians(0.64883707),
                                       **keywords):
@@ -80,10 +116,47 @@ class QubicAcquisition(Acquisition):
             The Full Width Half Maximum of the gaussian.
 
         """
-        return HealpixConvolutionGaussianOperator(self.instrument.sky.nside,
-                                                  fwhm=fwhm, **keywords)
+        return HealpixConvolutionGaussianOperator(fwhm=fwhm, **keywords)
 
-    def get_projection_peak_operator(self, kmax=2):
+    def get_hwp_operator(self):
+        """
+        Return the rotation matrix for the half-wave plate.
+
+        """
+        return Rotation3dOperator('X', -4 * self.pointing.angle_hwp,
+                                  degrees=True)
+
+    def get_polarizer_operator(self):
+        """
+        Return operator for the polarizer grid.
+
+        """
+        if not self.instrument.sky.polarized:
+            return HomothetyOperator(1 / self.instrument.detector.ngrids)
+
+        if self.instrument.detector.ngrids == 1:
+            return DenseOperator([[0.5, 0.5, 0],
+                                  [0.5,-0.5, 0]])
+
+        grid = self.instrument.detector.packed.quadrant // 4
+        z = np.zeros(self.get_ndetectors())
+        data = np.array([z + 0.5, 0.5 - grid, z]).T[:, None, :]
+
+        def direct(x, y):
+            np.sum(x * data, axis=-1, out=y)
+
+        def transpose(x, y):
+            np.multiply(x[..., None], data, out=y)
+
+        shapeout = self.get_ndetectors(), self.get_nsamples()[0]
+        shapein = shapeout + (3,)
+
+        from pyoperators import Operator
+        return Operator(direct=direct, transpose=transpose, dtype=float,
+                        shapein=shapein, shapeout=shapeout,
+                        flags='real, linear')
+
+    def get_projection_peak_operator(self, kmax=2, rotation=None):
         """
         Return the projection operator for the peak sampling.
 
@@ -92,9 +165,13 @@ class QubicAcquisition(Acquisition):
         kmax : int, optional
             The diffraction order above which the peaks are ignored.
             For a value of 0, only the central peak is sampled.
+        rotation : array (npointings, 3, 3)
+            The Instrument-to-Reference rotation matrix
 
         """
-        return self.instrument.get_projection_peak_operator(self.pointing,
+        if rotation is None:
+            rotation = self.get_rotation_g2i()
+        return self.instrument.get_projection_peak_operator(rotation,
                                                             kmax=kmax)
 
     @classmethod
